xquery version "3.1";

module namespace presort="http://fontane-nb.dariah.eu/presort";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace test="http://exist-db.org/xquery/xqsuite";

import module namespace functx="http://www.functx.com";
import module namespace simpleHelpers="http://fontane-nb.dariah.eu/teisimplehelpers" at "teisimplehelpers.xqm";


(:~
 : Marks all nodes that are part of an interlinear addition.
 :
 : @author Michelle Weidling
 : @param $node The current node
 : @return The processed node
 :)
declare function presort:prepare($nodes as node()*) as node()* {
    for $node in $nodes
      return
        typeswitch ($node)
        case text() return
            $node

        case comment() return
            if(matches($node/string(), "rotate")) then
                $node
            else
                ()

        default return
            if($node/preceding-sibling::*[self::tei:addSpan][1][@place = 'interlinear'][@prev or @next]
            and not($node[self::tei:anchor])) then
                presort:mark-interlinear-node-to-be-moved($node)
            else
                element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                    $node/@*,
                    presort:prepare($node/node())
                }

};


(:~
 : Interlinear additions can be part of a virtual aggregation. Since their
 : start is marked by tei:addSpan and their end by tei:anchor, all nodes inbetween
 : have to be moved when the right chronology is established, too. To make this
 : easier, the respective nodes are marked with an attribute.
 :
 : All other nodes are simply copied.
 :
 : @author Michelle Weidling
 : @param $node The current node
 : @return The processed node
 :)
declare function presort:mark-interlinear-node-to-be-moved($node as element())
as element() {
    let $addSpan := $node/preceding-sibling::*[self::tei:addSpan][1][@place = 'interlinear'][@prev or @next]
    let $spanTo := substring-after($addSpan/@spanTo, "#")
    let $anchor := $addSpan/following::tei:anchor[@xml:id = $spanTo]
    let $nodes-inbetween := $addSpan/following-sibling::*[. << $anchor]

    return
        element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
            $node/@*,
            if(functx:is-node-in-sequence-deep-equal($node, $nodes-inbetween)) then
                attribute type {"interlinear"}
            else
                (),
            $node/node()
        }
};


(:~
 : Creates a new encoding structure for tei:metamark[@function = "integrate"]
 : that makes further processing of them easier. All nodes relevant for the
 : integration brackets are summarized in a tei:seg[@type = 'integration'].
 :
 : @author Michelle Weidling
 : @param $node The current node
 : @return The processed node
 :)
declare function presort:sort-integrations($nodes as node()*) as node()* {
    for $node in $nodes return
    if($node/@xml:id) then
        let $id := $node/@xml:id
        let $metamarks := $node/ancestor::*[last()]//tei:metamark[@function = "integrate"]
        let $linking-node-corresp := $metamarks[substring-after(@corresp, "#") = $id]
        let $linking-node-target := $metamarks[substring-after(@target, "#") = $id]

        return
            if($linking-node-corresp and not($linking-node-target)) then
                let $integration-target-id := replace($linking-node-corresp/@target, "#", "")
                let $integration-comment := $linking-node-corresp/ancestor::*[last()]//*[@xml:id = $integration-target-id]
                return
                    element tei:seg {
                        attribute type {"integration"},
                        $integration-comment,
                        $linking-node-corresp,
                        presort:keep-node-integrations($node)
                    }
            else if($linking-node-target) then
                ()
            else
                presort:keep-node-integrations($node)

    else if($node[self::tei:metamark[@function = "integrate"]]) then
        ()
    else if($node[self::text() or self::comment()]) then
        $node
    else
        presort:keep-node-integrations($node)
};


(:~
 : An auxiliary function that copies a given node while sorting the integrations.
 :
 : @author Michelle Weidling
 : @param $node The current node
 : @return A copy of the given node
 :)
declare function presort:keep-node-integrations($node as node()) as node()* {
    element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
        $node/@*,
(:        $node/(@* except (@prev, @next)), (: attrs only visible for debugging :):)
        presort:sort-integrations($node/node())
    }
};
