xquery version "3.1";

(:~
 : This module handles the creation of the XHTML serialization of the edited text.
 : On the Fontane website each notebook is provided as a so called "Lesefassung"
 : that contains only the contemporary, definitive text as well as some styling
 : information such as pencil colours and different fonts.
 :
 :)

module namespace simple2xhtml="http://fontane-nb.dariah.eu/simple2xhtml";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace xhtml="http://www.w3.org/1999/xhtml";

import module namespace config="http://textgrid.de/ns/SADE/config" at "../../config/config.xqm";

declare function simple2xhtml:main($nodes as node()*, $uri as xs:string) {
    let $xhtml := element xhtml:div {simple2xhtml:recursion($nodes//tei:text)}
    let $store := xmldb:store($config:data-root || "/print/xhtml/", $uri || ".html", $xhtml)
    return
        $xhtml
};

(:~
 : Tidy up all that messy white spaces.
 : @TODO further optimization of tei2simple
 :)
declare function simple2xhtml:fix-whitespaces($nodes as node()*) as node()* {
    let $string :=
        serialize($nodes)
(:        => replace("[\t\n]", ""):)
(:        => replace("[\s]+", " "):)
(:        => replace(" ,", ","):)
(:        => replace(" \.", "."):)

    return $string
};


(:~
 : The main transformation for XML to XHTML.
 :
 : @author Simon Sendler
 : @author Michelle Weidling
 : @param $node The current TEI node
 : @return The respective XHTML node
 :)
declare function simple2xhtml:recursion($nodes as node()*) as node()* {
    for $node in $nodes return
        typeswitch ($node)

        case text() return
            if (normalize-space($node) = "") then
                ()
            else
                element xhtml:span {
                    simple2xhtml:set-hs-info($node, ()),
                    replace($node, "@P", "")
                    => replace(" ,", ",")
                    => replace(" \?", "?")
                    => replace(" \.", ".")
                    => replace(" ;", ";")
                    => replace("@", " ")
                }

        case element(tei:body) return
            element xhtml:div {
                simple2xhtml:set-hs-info($node, ()),
                simple2xhtml:recursion($node/node())
            }

        case element(tei:abbr) return
            (text{" "},
            element xhtml:span {
                simple2xhtml:set-hs-info($node, "abbr"),
                simple2xhtml:recursion($node/node())
            })

        case element(tei:add) return
            if($node/@place = "above"
            and $node/preceding-sibling::*[1][self::tei:seg[@type = "multiphrase"]]) then
                let $corresp-id := replace($node/@corresp, "#", "")
                let $corresp-anchor := $node/root()//tei:anchor[@xml:id = $corresp-id]
                let $corresp-seg := $corresp-anchor/ancestor::tei:seg[@type = 'multiphrase']
                let $super := $node/root()//tei:seg[@type = "multiphrase"][@xml:id = replace($corresp-seg/@corresp, "#", "")]
                let $sub := $corresp-seg
                return
                    element xhtml:span {
                        simple2xhtml:set-hs-info($node, "multiphrase"),
                        element xhtml:span {
                            attribute class {"multiphrase-top"},
                            simple2xhtml:recursion($super/node())
                        },
                        element xhtml:span {
                            attribute class {"multiphrase-bottom"},
                            simple2xhtml:recursion($sub/node())
                        }
                    }
            else
                simple2xhtml:recursion($node/node())

        case element(tei:div) return
            if($node/@type = "toc") then
                if($node/@subtype = "Friedrich_Fontane") then
                    (element xhtml:span {
                        attribute class {"handshift"},
                        text{"<Schreiberhand Friedrich Fontane>"}
                    },
                    element xhtml:ul {
                        simple2xhtml:set-hs-info($node, "nb-toc-list"),
                        simple2xhtml:recursion($node/node())
                    },
                    element xhtml:span {
                        attribute class {"handshift"},
                        text{"<Schreiberhand Theodor Fontane>"}
                    })
                else
                    element xhtml:ul {
                        simple2xhtml:set-hs-info($node, "nb-toc-list"),
                        simple2xhtml:recursion($node/node())
                    }
            else
                element xhtml:div {
                    if($node/@type = "label") then
                        simple2xhtml:set-hs-info($node, "nb-label")
                    else
                        simple2xhtml:set-hs-info($node, ()),
                    simple2xhtml:recursion($node/node())
                }

        case element(tei:expan) return
            element xhtml:span {
                simple2xhtml:set-hs-info($node, "expan"),
                simple2xhtml:recursion($node/node())
            }

        case element(tei:front) return
            element xhtml:div {
                simple2xhtml:set-hs-info($node, "front-area"),
                simple2xhtml:recursion($node/node())
            }

        case element(tei:hi) return
            (text{" "},
            element xhtml:span {
                simple2xhtml:set-hs-info($node, "nb-underline"),
                simple2xhtml:recursion($node/node())
            })

        case element(tei:lb) return
            if ($node/@type="edited_text") then
                element xhtml:br {}
            else
                ()

        case element(tei:list) return
            (: this matches the FIRST element of the virtual aggregation :)
            if($node/@next and not($node/@prev)) then
                element xhtml:ul {
                    attribute class {"nb-list"},
                    simple2xhtml:apply-next-element($node)
                }
            (: this matches all OTHER elements of the virtual aggregation :)
            else if($node/@prev or $node/@next) then
                ()
            else
                element xhtml:ul {
                    attribute class {"nb-list"},
                    simple2xhtml:recursion($node/node())
                }

        case element(tei:item) return
            element xhtml:li {
                simple2xhtml:set-hs-info($node, ()),
                simple2xhtml:recursion($node/node())
            }

        case element(tei:rs) return
            let $id := $node/@ref => substring-after(":")
            let $index-link := "register.html?e=" || $id
            return
                element xhtml:a {
                    attribute href {$index-link},
                    attribute target {"_blank"},
                    simple2xhtml:set-hs-info($node, "nb-index-entry"),
                    simple2xhtml:recursion($node/node()),
                    if($node/@prev or $node[not(@prev or @next)]) then
                        (switch(substring-before($node/@ref, ":"))
                            case "psn" return
                                <i class="fa fa-user icon-invisible" aria-hidden="true"/>
                            case "plc" return
                                <i class="fa fa-map-marker icon-invisible" aria-hidden="true"/>
                            case "eve" return
                                <i class="fa fa-flag icon-invisible" aria-hidden="true"/>
                            case "org" return
                                <i class="fa fa-university icon-invisible" aria-hidden="true"/>
                            case "wrk" return
                                <i class="fa fa-book icon-invisible" aria-hidden="true"/>
                            default return ()

    (:                        let $first-char := $node/following::text()[1]/substring(., 1, 1):)
    (:                        return:)
    (:                        if(matches($first-char, "[\.,)(\?!]")) then:)
    (:                            ():)
    (:                        else:)
    (:                            text{" "}:)
                        )
                    else
                        ()
                }

        case element(tei:head) return
            switch($node/@subtype)
                case "sub" return
                    element xhtml:h5 {
                        simple2xhtml:set-classes($node),
                        simple2xhtml:recursion($node/node())
                    }

                case "chapter" return
                    element xhtml:h4 {
                        simple2xhtml:set-classes($node),
                        simple2xhtml:recursion($node/node())
                    }

                case "section" return
                    element xhtml:h3 {
                        simple2xhtml:set-classes($node),
                        simple2xhtml:recursion($node/node())
                    }

                default return
                    element xhtml:h2 {
                        simple2xhtml:set-classes($node),
                        simple2xhtml:recursion($node/node())
                    }

        case element(tei:milestone) return
            if($node/@unit = "line") then
                let $next-char := substring($node/following::text()[1], 1, 1)
                return
                    if(matches($next-char, "[\.\)\?,;!]")
                    and not(ends-with($node/preceding::text()[1], "@P"))) then
                        ()

                    else if(ends-with($node/preceding::text()[1], "@P")
                    or ends-with($node/preceding::text()[1], "-")) then
                        ()

                    else if($node/preceding-sibling::*[1][self::tei:rs[concat('#', @xml:id) = $node/following-sibling::*[2][self::tei:rs]/@prev]]) then
                        ()

                    else
                        text{" "}

            else if($node/@unit = "start-lg") then
                let $lines := $node/following-sibling::tei:l[. << $node/following-sibling::tei:milestone[@unit = "end-lg"][1]]
                return
                    element xhtml:div {
                        attribute class {"nb-verses"},
                        for $line in $lines return
                            element xhtml:div {
                                attribute class {"nb-verse"},
                                simple2xhtml:recursion($line/node())
                            }
                    }

            else if($node/@unit = "paragraph") then
                (element xhtml:br {},
                element xhtml:div {
                    attribute class {"nb-paragraph"}
                })

            else
                ()


        (: lines are handled together with tei:milestone[@unit = "start-lg"]. all
        other verses are ignored. :)
        case element(tei:l) return
            ()

        case element(tei:ab) return
            if($node/@type = "sketch"
            and $node/descendant::tei:figure) then
                element xhtml:table {
                    attribute class {"nb-sketch"},
                    element xhtml:tr {
                        simple2xhtml:recursion($node/node())
                    }
                }

            else if($node/@type = ("caret")) then
                ()

            else if($node/@type = "footnote-mark") then
                element xhtml:seg {
                    attribute class {"nb-" || $node/@type},
                    simple2xhtml:recursion($node/node())
                }

            else if($node/@type = "short-paragraph-line") then
                element xhtml:div {
                    attribute class {$node/@type}
                }

            else if($node/@type = "paragraph" and $node/string = "z") then
                (element xhtml:br {},
                    element xhtml:div {
                    attribute class {"nb-paragraph"}
                })

            else
                simple2xhtml:recursion($node/node())

        case element(tei:figure) return
            let $captions-before :=
                $node/preceding-sibling::tei:seg[@type = "caption"][ancestor::tei:ab = $node/ancestor::tei:ab]
            let $captions-after :=
                $node/following-sibling::tei:seg[@type = "caption"][ancestor::tei:ab = $node/ancestor::tei:ab]
            return
                (:  make captions left of sketch:)
                (if(exists($captions-before)) then
                    simple2xhtml:make-img-captions($captions-before, "right-aligned")
                else
                    (),

                element xhtml:td {
                    attribute class {"nb-img"},
                    if($node/@rotate) then
                        attribute style {concat("width: ", $node/parent::*/@height, "; height: ", $node/parent::*/@width, ";")}
                    else
                        (),
                    simple2xhtml:make-img($node, "block")
                },

                (: make captions right of sketch:)
                if(exists($captions-after)) then
                    simple2xhtml:make-img-captions($captions-after, "left-aligned")
                else
                    ())

        case element(tei:seg) return
            if($node/@type = ("caption", "editorial-label", "multiphrase")) then
                ()

            else if($node[@rendition = ("vertical-align:super", "vertical-align:sub")
            and
                (following-sibling::*[1][self::tei:g[@ref ="#hb"]]
                or preceding-sibling::*[1][self::tei:g[@ref ="#hb"]])
            ]) then
                ()

            else if($node/@type = "integration") then
                simple2xhtml:make-integration($node)

            else
                element xhtml:span {
                    simple2xhtml:set-hs-info($node, "nb-seg"),
(:                    attribute style {$node/@rendition},:)
                    simple2xhtml:recursion($node/node())
                }

        case element(tei:unclear) return
            element xhtml:span {
                attribute class {"unclear"},
                simple2xhtml:recursion($node/node())
            }

        case element(tei:date) return
            (simple2xhtml:set-whitespace-before($node),
            element xhtml:span {
                simple2xhtml:set-hs-info($node, "nb-date"),
                simple2xhtml:recursion($node/node()),
            simple2xhtml:set-whitespace-after($node)
            })

(:little helper, not yet specified for transformation:)

        case element(tei:index) return
            ()

        case element(tei:note) return
            if($node/@type = "editorial") then
                element xhtml:span {
                    attribute class {"editorial-note"},
                    simple2xhtml:recursion($node/node())
                }
            else if($node/@subtype = "footnote") then
                element xhtml:div {
                    attribute class {"nb-footnote"},
                    simple2xhtml:recursion($node/node())
                }
            else
                ()

        case element(tei:g) return
            if($node/@ref = "#hb") then
                let $super := $node/preceding-sibling::tei:seg[@rendition = "vertical-align:super"][1]
                let $sub := $node/following-sibling::tei:seg[@rendition = "vertical-align:sub"][1]
                return
                    element xhtml:span {
                        simple2xhtml:set-hs-info($node, "fraction"),
                        element xhtml:span {
                            simple2xhtml:set-hs-info($node, "fraction-top"),
                            simple2xhtml:recursion($super/node())
                        },
                        element xhtml:span {
                            simple2xhtml:set-hs-info($node, "fraction-bottom"),
                            simple2xhtml:recursion($sub/node())
                        }
                    }

            else if($node/@ref = "#rth") then
                element xhtml:span {
                    attribute class {'g rth'},
                    <xhtml:div class="hover">Reichstaler</xhtml:div>,
                        <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 82 107">
                        {element path {
                        attribute fill {"none"},
                        attribute stroke {"darkslategrey"},
                        attribute stroke-width {"5"},
                        attribute d {
                        "M 24.73,35.82" ||
                        " C 24.73,35.82 30.36,30.73 32.00,26.55" ||
                        " 46.73,45.27 80.55,37.45 70.00,38.18" ||
                        " 16.73,50.73 -2.55,99.09 19.09,101.82" ||
                        " 52.00,103.45 57.09,80.00 42.91,80.91" ||
                        " 11.82,83.64 35.45,105.45 52.00,99.64" }}}
                        </svg>
                }

            else
                ()


        default return
            simple2xhtml:recursion($node/node())
};

(:~
 : Sets classes necessary for CSS. This encompassing mainly information about
 : the writing medium as well as heading sizes.
 :
 : @author Michelle Weidling
 : @param $node The current element
 : @return
 :)
declare function simple2xhtml:set-classes($node as element(*)) as attribute() {
    if($node/@rend = "align(center)"
    and $node/ancestor::tei:div[@type = "label"]) then
        simple2xhtml:set-hs-info($node, "nb-head nb-sub-head nb-centered nb-label-head")

    else if($node/@rend = "align(center)") then
        simple2xhtml:set-hs-info($node, "nb-head nb-sub-head nb-centered")

    else if($node/ancestor::tei:div[@type = "label"]) then
        simple2xhtml:set-hs-info($node, "nb-head nb-sub-head nb-label-head")

    else
        simple2xhtml:set-hs-info($node, "nb-head nb-sub-head")
};


(:~
 : Creates a xhtml:img element with all information necessary for getting the
 : right TBLE snippet and displaying it correctly.
 :
 : @author Michelle Weidling
 : @param $node The current tei:figure
 : @param $mode "right-aligned", "block" or "inline"
 : @return A complete xhtml:img element
 :)
declare function simple2xhtml:make-img($node as element(tei:figure),
$mode as xs:string) as element(xhtml:img) {
    let $width := "width:" || $node/ancestor::tei:ab[@type = "sketch"]/@width || "; "
    let $height := "height:" || $node/ancestor::tei:ab[@type = "sketch"]/@height || ";"
    let $rotation :=
        if($node/@rotate) then
            "transform: rotate(" || $node/@rotate || "deg);"
        else
            ""

    return
        element xhtml:img {
            attribute src {$node/@href},
            attribute alt {$node//tei:figDesc/string()},
(:            if($mode = "block") then:)
(:                attribute class {"nb-block-img"}:)
(:            else:)
                attribute class {"nb-block-img"},
            attribute style {$width || $height || $rotation}
        }
};


(:~
 : Creates the captions for a tei:figure.
 :
 : @author Michelle Weidling
 : @param The given captions as tei:seg[@type = 'caption']
 : @param $align "right-aligned" or "left-aligned"
 : @return An element entry containing the caption contents
 :)
declare function simple2xhtml:make-img-captions($nodes as element(tei:seg)+,
$align as xs:string) as element(xhtml:td) {
    element xhtml:td {
        attribute class {"nb-caption-data"},
        for $node in $nodes return
            element xhtml:span {
                if($align = "right-aligned") then
                    simple2xhtml:set-hs-info($node, "nb-caption nb-caption-ra")
                else
                    simple2xhtml:set-hs-info($node, "nb-caption nb-caption-la"),
                simple2xhtml:recursion($node/node())
            }
    }
};


(:~
 : Sets a white space before a node.
 :
 : @author Michelle Weidling
 : @param The current node
 : @return One or zero white space(s)
 :)
declare function simple2xhtml:set-whitespace-before($node as element(*)) as text()? {
    let $prev-text := $node/preceding::text()[1]
    let $last-char := substring($prev-text, string-length($prev-text), 1)
    return
        if(matches($last-char, "[(†]")) then
            ()
        else
            text{" "}
};


(:~
 : Sets a white space after a node.
 :
 : @author Michelle Weidling
 : @param The current node
 : @return One or zero white space(s)
 :)
declare function simple2xhtml:set-whitespace-after($node as element(*)) as text()? {
    let $next-text := $node/following::text()[1]
    let $first-char := substring($next-text, 1, 1)
    return
        if(matches($first-char, "[“\.,\?!;\)]")) then
            ()
        else
            text{" "}
};


(:~
 : Transforms all element of a virtual aggregation to XHTML.
 :
 : @author Michelle Weidling
 : @param The current node
 : @return The processed node
 :)
declare function simple2xhtml:apply-next-element($node as node()*)
as node()* {
    (: exit point. applies the contents of the current element. :)
    if(not($node/@next)) then
        simple2xhtml:recursion($node/node())

    (: nodes that have a @prev and/or a @next:)
    else
        (: because sort.xqm has been successful we already know that the
        next sibling is the one referenced in @next :)
        let $next-node := $node/following-sibling::*[1]
        return
            (simple2xhtml:recursion($node/node()),
            simple2xhtml:apply-next-element($next-node))
};


(:~
 : Creates a class attribute for the resulting XHTML element containing information
 : about the current writing medium as well as other CSS classes.
 :
 : @author Michelle Weidling
 : @param $node The current node
 : @param $attributes Further class attributes that the resulting XHTML element should have
 : @return A class attribute with all relevant entries
 :)
declare function simple2xhtml:set-hs-info($node as node(), $attributes as xs:string*)
as attribute() {
    let $prev-hs := $node/preceding::tei:milestone[@unit = "handshift"][1]
    let $classes :=
        if($attributes) then
            $prev-hs/@rend || " " || $attributes
        else
            $prev-hs/@rend

    return
        attribute class {$classes}
};


declare function simple2xhtml:make-integration($node as element(tei:seg)) {
    let $bracket := $node//tei:ab[@function = "integrate"]

    (: target is the side where the pointy end of the bracket goes.
    corresp is the side where bracket faces to.
    example: target { corresp :)
    let $target-id := replace($bracket/@target, "#", "")
    let $target := $node//*[@xml:id = $target-id]
    let $corresp-id := replace($bracket/@corresp, "#", "")
    let $corresp := $node//*[@xml:id = $corresp-id]

    let $lines-target := count($target//tei:milestone[@unit = "line"])
    let $lines-corresp := count($corresp//tei:milestone[@unit = "line"])
    let $max-number-of-lines :=
        if($lines-target gt $lines-corresp) then
            $lines-target
        else
            $lines-corresp
    let $bracket-size := 12 * ($max-number-of-lines + 1)


    return
        element xhtml:table {
            attribute class {"integration"},
            element xhtml:tr {
                element xhtml:td {
                    attribute class {"integration-left"},
                    if($bracket/@rend = "bracket_right") then
                        simple2xhtml:recursion($corresp/node())
                    else if($bracket/@rend = "bracket_left") then
                        simple2xhtml:recursion($target/node())
                    else
                        ()
                },
                element xhtml:td {
                    attribute class {"integration-bracket"},
                    attribute style {"font-size: " || $bracket-size || "pt;"},
                    if($bracket/@rend = "bracket_right") then
                        text{"}"}
                    else if($bracket/@rend = "bracket_left") then
                        text{"{"}
                    else
                        ()
                },
                element xhtml:td {
                    attribute class {"integration-right"},
                    if($bracket/@rend = "bracket_right") then
                        simple2xhtml:recursion($target/node())
                    else if($bracket/@rend = "bracket_left") then
                        simple2xhtml:recursion($corresp/node())
                    else
                        ()
                }
            }
        }
};
